CREATE OR REPLACE FUNCTION describe_association_rules(varchar,int,numeric(5,2)) RETURNS SETOF rules AS '
DECLARE
	associator	ALIAS FOR $1;
	size		ALIAS FOR $2;
	conf_min	ALIAS FOR $3;

	cont_reg	integer;
	ini		integer;
	fin		integer;
	val_ext		integer;
	fin_regla	integer;
	
	sop_a		numeric;
	sop_r		numeric;
	confianza	numeric(5,2);
	
	qry_sql		text DEFAULT '''';
	cad_att		text DEFAULT NULL;
	cad_var		text DEFAULT NULL;
	cad_join	text DEFAULT NULL;
	
	rec_at		record;
	rec_sop		record;
	rec_rules	rules%ROWTYPE;
	
BEGIN
	--****	VALIDACION CONFIANZA
	
	IF conf_min < 0 or conf_min > 100 THEN
		RAISE EXCEPTION ''CONFIANZA % no se encuentra en el rango de porcentaje'',conf_min;
		RETURN; 
	END IF;
	
	IF size < 0 THEN
		RAISE EXCEPTION ''SIZE acepta unicamente valores positivos'';
		RETURN; 
	END IF;
	--****

	
	--****	CREA TABLA TEMPORAL DE ASSOCIATOR dar_asso
	
	qry_sql := ''SELECT attname,attnum,relnatts FROM pg_class, pg_attribute'';
	qry_sql := qry_sql || '' WHERE pg_class.relname = ''''''||associator||'''''''';
	qry_sql := qry_sql || '' AND pg_attribute.attrelid = pg_class.oid'';
	qry_sql := qry_sql || '' AND attnum > 0 ORDER BY attnum'';
	
	FOR rec_at IN EXECUTE qry_sql LOOP
	
		IF rec_at.attnum < rec_at.relnatts THEN
			IF cad_var ISNULL THEN cad_var := rec_at.attname||'' bpchar'';
			ELSE cad_var := cad_var ||'' , ''|| rec_at.attname||'' bpchar'';
			END IF;
		ELSE
			IF rec_at.attname != ''soporte'' THEN
				EXECUTE ''ALTER TABLE ''||associator||'' RENAME COLUMN ''||rec_at.attname||'' TO soporte'';
			END IF;
		END IF;
		
	END LOOP;
	cad_var := cad_var ||'' , soporte int8'';
	
	EXECUTE ''CREATE TABLE dar_asso (''||cad_var||'')''; 
	EXECUTE ''INSERT INTO dar_asso SELECT * FROM ''||associator;
	
	qry_sql := ''SELECT attname FROM pg_class, pg_attribute'';
	qry_sql := qry_sql || '' WHERE pg_class.relname = ''''dar_asso'''''';
	qry_sql := qry_sql || '' AND pg_attribute.attrelid = pg_class.oid'';
	qry_sql := qry_sql || '' AND attnum > 0 AND attnum < relnatts ORDER BY attnum'';
	
	FOR rec_at IN EXECUTE qry_sql LOOP
	
		EXECUTE ''UPDATE dar_asso SET ''||rec_at.attname||'' = ''''-'''' where ''||rec_at.attname||'' isnull'';
		
	END LOOP;
	--****
	
	
	--****	CREA TABLA TEMPORAL DE REGLAS dar_rcan DE assoc_rules()
	
	EXECUTE ''CREATE TABLE dar_rcan AS SELECT * FROM assoc_rules(''''''||associator||'''''',''||size||'')'';
	
	qry_sql := ''SELECT relname, attname FROM pg_class, pg_attribute'';
	qry_sql := qry_sql || '' WHERE pg_class.relname = ''''dar_rcan'''''';
	qry_sql := qry_sql || '' AND pg_attribute.attrelid = pg_class.oid'';
	qry_sql := qry_sql || '' AND attnum > 0 AND attnum < (relnatts - 3) ORDER BY attnum'';
	
	FOR rec_at IN EXECUTE qry_sql LOOP
		
		IF cad_att ISNULL THEN
			cad_att := rec_at.relname||''.''||rec_at.attname;
			cad_join := ''dar_asso.''||rec_at.attname||'' = dar_rcan.''||rec_at.attname;
		ELSE
			cad_att := cad_att ||'' , ''|| rec_at.relname||''.''||rec_at.attname;
			cad_join := cad_join ||'' AND ''|| ''dar_asso.''||rec_at.attname||'' = dar_rcan.''||rec_at.attname;
		END IF;
		
		EXECUTE ''UPDATE dar_rcan SET ''||rec_at.attname||'' = ''''-'''' where ''||rec_at.attname||'' isnull'';
		
	END LOOP;
	--****
	
	
	--**** CREA TABLA TEMPORAL DE REGLAS CANDIDATAS dar_rcanf
	
	qry_sql := ''CREATE TABLE dar_rcanf AS SELECT ''||cad_att||'', dar_rcan.n_regla, dar_rcan.implica, dar_asso.soporte, 				dar_rcan.confianza FROM dar_rcan, dar_asso WHERE ''||cad_join||'' ORDER BY dar_rcan.n_regla, dar_rcan.implica'';
	EXECUTE qry_sql;
	--****
	
	
	--****	GENERA REGLAS FINALES DE ASOCIACION
	
	SELECT INTO cont_reg count(*) FROM dar_rcanf;
	SELECT INTO ini oid as cod FROM dar_rcanf;
	fin := ini + (cont_reg - 1);
	val_ext := (((2 ^ size) - 2) * 2) - 1;
	
	qry_sql := ''SELECT soporte FROM ''||associator||'', dar_size WHERE dar_size.fq = ''||size||'' 
			AND ''||associator||''.oid = dar_size.codigo'';
	
	FOR rec_sop IN EXECUTE qry_sql LOOP
	
		sop_r := rec_sop.soporte;
		fin_regla := ini + val_ext;
		
		WHILE ini <= fin_regla and ini <= fin LOOP
	
			SELECT INTO sop_a soporte FROM dar_rcanf WHERE oid = ini; 
			confianza = (sop_r / sop_a) * 100;
						
			IF confianza >= conf_min THEN
							
				SELECT INTO rec_rules * FROM dar_rcanf WHERE oid = ini; 
				rec_rules.confianza := confianza;
				RETURN NEXT rec_rules;
		
				SELECT INTO rec_rules * FROM dar_rcanf WHERE oid = (ini+1); 
				RETURN NEXT rec_rules;
						
			END IF;
			
			ini := ini + 2;
			
		END LOOP;
		
	END LOOP;
	--****
	
	
	--**** ELIMINA TABLAS TEMPORALES
	
	EXECUTE ''DROP TABLE dar_size'';
	EXECUTE ''DROP TABLE dar_rasso'';
	EXECUTE ''DROP TABLE dar_asso'';
	EXECUTE ''DROP TABLE dar_rcan'';
	EXECUTE ''DROP TABLE dar_rcanf'';
	--****
	RETURN;
END;'
LANGUAGE 'plpgsql' WITH (isstrict);
