#include "defs.h"
#include "busqueda.h"

/* Funcion que busca en la tabla mate_entro la tupla que 
 * cumpla las condiciones dadas en el vector cad. Con la tupla seleccionada
 * calcula la entropia y ganancia parcial, y en el vector decision(vecdec)
 * asigna el atributo que tine mayor ganancia y los valores de entropia.
 */
void busqueda(int vecdec[],text *relname,int numAttr,int NCLASES,int numClases[],int **matriz)
{	
	int i,j,k,aux,nclases,atteq,sum;
	int *cad;
	cad = nuevoArreglo(int, numAttr);
	
	float *tupla;
	float gainpar=0,gain=9999;
	
	RangeVar   *relrv;
	Relation	rel;
	HeapScanDesc scan;
	TupleDesc	tupdesc;

	relrv = makeRangeVarFromNameList(textToQualifiedNameList(relname,"gain"));
	rel = heap_openrv(relrv, AccessShareLock);
	tupdesc = rel->rd_att;
	tupla=nuevoArreglo(float, tupdesc->natts);
	scan = heap_beginscan(rel, SnapshotAny, 0, NULL);
	
	for(i=0; i<numAttr; i++) {
		
		int s=0;
		for(aux=0; aux<numAttr && matriz[0][aux]!=-1; aux++){
			if(i == matriz[0][aux]){
				s=1;
				break;
			}
		}
		if(s ==1)
			continue;
		
		nclases=numClases[i];
		int *sumpar;
		float *entpar;
		sumpar = nuevoArreglo(int, nclases);
		entpar = nuevoArreglo(float, nclases);
		bzero(sumpar, nclases * sizeof (int));
		bzero(entpar, nclases * sizeof (float));
		int vecdecpar[nclases];
		sum=0;
		gainpar=0;
		
		for(aux=0; aux<numAttr; aux++) 
			cad[aux]=-1;
		for(aux=0; aux<nclases; aux++) 
			vecdecpar[aux]=-999;

		for(j=0; j<nclases; j++) {
			for(k=0; k<NCLASES; k++) {
				cad[i]= j;
				cad[numAttr]= k;
				for(aux=0; aux<numAttr && matriz[0][aux]!=-1; aux++){
					cad[matriz[0][aux]]=matriz[1][aux];
				}
				while (leerTupla(tupla,&scan,tupdesc)){
					atteq=0;
					for(aux=0; aux<=numAttr; aux++){
						if(cad[aux]==tupla[aux])
							atteq++;
						else
							break;
					}
					if(atteq>numAttr){
						if(tupla[numAttr+2]!=0)
							vecdecpar[j]=tupla[numAttr+2];
						else
							vecdecpar[j]=tupla[numAttr];				
						//elog(INFO,"%f %f %f %f %f count=%f entro=%f",tupla[0],tupla[1],tupla[2],tupla[3],tupla[4],tupla[numAttr+1],tupla[numAttr+2]);
						sumpar[j]=sumpar[j]+tupla[numAttr+1];
						entpar[j]=entpar[j]+(tupla[numAttr+2]/1000);//div entre 1000
					}
				}
			}
			sum=sum+sumpar[j];
		}
		for(aux=0; aux<j; aux++)
			gainpar = gainpar + (((float)sumpar[aux]/(float)sum)*(float)entpar[aux]);
		//elog(INFO,"gain=%f",gainpar);
		if(gainpar < gain){
			//elog(INFO,"Paso");
			gain= gainpar;
			vecdec[0] = i;
			for(aux=0;aux<numClases[i];aux++)
				vecdec[aux+1]=vecdecpar[aux];
		}		
		liberar(sumpar);
		liberar(entpar);
	}
	//elog(INFO,"FIN");
				
	heap_endscan(scan);
	heap_close(rel, AccessShareLock);
	
	liberar(tupla);
	liberar(cad);
}

/* Funcion que abre la tabla mate_ento y retorna cada uno de los registros
 * en un vector(tupla)
 */
int leerTupla( float tupla[],HeapScanDesc *scan,TupleDesc	tupdesc){
	HeapTuple tuple;
	
	if( (tuple = heap_getnext(*scan, ForwardScanDirection)) != NULL ) {
		int i;
        Datum val;        
        bool isnull;
		for( i=0; i<=tupdesc->natts; i++ ) {
        	val = heap_getattr( tuple, i+1, tupdesc,&isnull );
        	if (i<tupdesc->natts)
	            tupla[i] = (float)DatumGetInt16( val );
        }
	return 1;  
	}	
	return 0;
}

/* Funcion que abre la tabla "mate_discret" y retorna el número de atributos(numAttr),
 * la cantidad de calses decision del atributo decision (NCLASES), 
 * y los clases de los atributos condicion(numClases[])
 */
void setDatos(int *numAttr,int *NCLASES,int numClases[]){
	int i=0;
	
	RangeVar   *relrv;
	Relation	rel;
	HeapScanDesc scan;
	TupleDesc	tupdesc;
	HeapTuple	tuple;

	Datum		attr;
	bool		isnull;
	
	List		*result = NIL;
	char *relname = "mate_values";
	
	result = lappend( result, makeString(pstrdup(relname)));
	relrv = makeRangeVarFromNameList(result);
	rel = heap_openrv(relrv, AccessShareLock);
	scan = heap_beginscan(rel, SnapshotAny, 0, NULL);
	tupdesc = rel->rd_att;
	
	while ((tuple = heap_getnext(scan, ForwardScanDirection)) != NULL) {	
		attr = heap_getattr(tuple,tupdesc->natts,tupdesc,&isnull);
		if (!isnull || DatumGetInt16(attr)!=0){
			numClases[i]=attr;
			//elog(INFO,"numClases[%d]=%d",i,numClases[i]);
			i++;
		}
	}
	
	heap_endscan(scan);
	heap_close(rel, AccessShareLock);
	
	*numAttr=i-1;
	*NCLASES=numClases[i-1];
}

